@@ -10,18 +10,27 @@ import os
 import re
 import warnings
 
-from . import alg_lists
-from . import validation  # moved out of the way to not obscure the flow
-
+from . import (
+    alg_lists,
+    validation,  # moved out of the way to not obscure the flow
+)
 
 # Defaults of integer property values (doubles as an allowlist)
 
-INT_DEFAULTS = {k: 0 for k in (
+INT_DEFAULTS = dict.fromkeys((
     'arbitrary_dh_groups',
     'min_dh_size', 'min_dsa_size', 'min_rsa_size',
     'sha1_in_certs',
-    'ssh_certs', 'ssh_etm',
-)}
+    'ssh_certs',
+), 0)
+
+
+# For enum values, first value works as default,
+
+ENUMS = {
+    'etm': ('ANY', 'DISABLE_ETM', 'DISABLE_NON_ETM'),
+    '__ems': ('DEFAULT', 'ENFORCE', 'RELAX'),  # FIPS/NO-ENFORCE-EMS
+}
 
 
 # Scopes (`@!ipsec`) and matching them
@@ -78,7 +87,7 @@ class ScopeSelector:
                                          original_pattern=self.pattern)
 
     def __str__(self):
-        return f'<ScopeSelector pattern={repr(self.pattern)}>'
+        return f'<ScopeSelector pattern={self.pattern!r}>'
 
     def matches(self, scopes):
         """
@@ -101,14 +110,14 @@ class ScopeSelector:
 # Operations: interpreting right hand sides of (sub)policy files
 
 class Operation(enum.Enum):
-    """
-    An operation that comes with the right-hand value of the directive.
-    """
+    """An operation that comes with the right-hand value of the directive."""
+
     RESET = 1     # cipher =
     PREPEND = 2   # cipher = +NULL
     APPEND = 3    # cipher = NULL+
     OMIT = 4      # cipher = -NULL
     SET_INT = 5   # sha1_in_certs = 0; setting to something that's all digits
+    SET_ENUM = 6  # __ems = ENFORCE
 
     def __repr__(self):  # to unify the output between Python versions
         return f'Operation.{self.name}'
@@ -129,30 +138,37 @@ def parse_rhs(rhs, prop_name):
     >>> parse_rhs('+*DES-CBC', 'cipher')
     [(Operation.PREPEND, 'DES-CBC'),
      (Operation.PREPEND, '3DES-CBC')]
+    >>> parse_rhs('ENFORCE', '__ems')
+    [(Operation.SET_ENUM, 'ENFORCE')]
     """
     def differential(v):
-        return v.startswith('+') or v.endswith('+') or v.startswith('-')
+        return v.startswith(('+', '-')) or v.endswith('+')
 
     if rhs.isdigit():
         if prop_name not in alg_lists.ALL and prop_name in INT_DEFAULTS:
             return [(Operation.SET_INT, int(rhs))]
-        elif prop_name in alg_lists.ALL:
+        if prop_name in alg_lists.ALL or prop_name in ENUMS:
             raise validation.rules.NonIntPropertyIntValueError(prop_name)
-        else:
-            assert prop_name not in alg_lists.ALL
-            assert prop_name not in INT_DEFAULTS
-            # pass for now, it's gonna be caught as non-existing algclass
+        assert prop_name not in alg_lists.ALL
+        assert prop_name not in INT_DEFAULTS
+        assert prop_name not in ENUMS
+        # pass for now, it's gonna be caught as non-existing algclass
     else:
         if prop_name in INT_DEFAULTS:
             raise validation.rules.IntPropertyNonIntValueError(prop_name)
+        if prop_name in ENUMS:
+            if rhs not in ENUMS[prop_name]:
+                raise validation.rules.BadEnumValueError(prop_name, rhs,
+                                                         ENUMS[prop_name])
+            return [(Operation.SET_ENUM, rhs)]
 
     values = rhs.split()
 
     if not any(differential(v) for v in values):  # Setting something anew
-        values = sum([alg_lists.glob(v, prop_name) for v in values], [])
+        values = [x for v in values for x in alg_lists.glob(v, prop_name)]
         return ([(Operation.RESET, None)]
                 + [(Operation.APPEND, v) for v in values])
-    elif all(differential(v) for v in values):  # Modifying an existing list
+    if all(differential(v) for v in values):  # Modifying an existing list
         operations = []
         for value in values:
             if value.startswith('+'):
@@ -167,14 +183,14 @@ def parse_rhs(rhs, prop_name):
                 unglob = alg_lists.glob(value[1:], prop_name)
             operations.extend([(op, v) for v in unglob])
         return operations
-    else:  # Forbidden to mix them on one line
-        raise validation.rules.MixedDifferentialNonDifferentialError(rhs)
+    # Forbidden to mix them on one line
+    raise validation.rules.MixedDifferentialNonDifferentialError(rhs)
 
 
 # Directives: interpreting lines of (sub)policy files
 
 Directive = collections.namedtuple('Directive', (
-    'prop_name', 'scope', 'operation', 'value'
+    'prop_name', 'scope', 'operation', 'value',
 ))
 
 
@@ -244,10 +260,10 @@ def preprocess_text(text):
     """
     text = re.sub(r'#.*', '', text)
     text = text.replace('=', ' = ')
-    text = '\n'.join((l.strip() for l in text.split('\n')))
+    text = '\n'.join(l.strip() for l in text.split('\n'))
     text = text.replace('\\\n', '')
-    text = '\n'.join((l.strip() for l in text.split('\n')))
-    text = '\n'.join((re.sub(r'\s+', ' ', l) for l in text.split('\n')))
+    text = '\n'.join(l.strip() for l in text.split('\n'))
+    text = '\n'.join(re.sub(r'\s+', ' ', l) for l in text.split('\n'))
     text = re.sub('\n+', '\n', text).strip()
 
     if re.findall(r'\bprotocol\s*=', text):
@@ -275,16 +291,25 @@ def preprocess_text(text):
             'hash@DNSSec = -SHA1\nsign@DNSSec = -RSA-SHA1 -ECDSA-SHA1',
         'sha1_in_dnssec = 1':
             'hash@DNSSec = SHA1+\nsign@DNSSec = RSA-SHA1+ ECDSA-SHA1+',
+        'ssh_etm = 0': 'etm@SSH = DISABLE_ETM',
+        'ssh_etm = 1': 'etm@SSH = ANY',
+        'ssh_etm@([^= ]+) = 0':
+            'etm@\\1 = DISABLE_ETM',
+        'ssh_etm@([^= ]+) = 1':
+            'etm@\\1 = ANY',
     }
     for fr, to in PLAIN_REPLACEMENTS.items():
         regex = r'\b' + fr + r'\b'
-        if re.search(regex, text):
-            warnings.warn(PolicySyntaxDeprecationWarning(fr, to))
+        matches = {}
+        for match in re.finditer(regex, text):
+            matches[match.group(0)] = re.sub(regex, to, text)
+        for match_fr, match_to in matches.items():
+            warnings.warn(PolicySyntaxDeprecationWarning(match_fr, match_to))
         text = re.sub(regex, to, text)
 
     dtls_versions = list(alg_lists.DTLS_PROTOCOLS[::-1])
     while dtls_versions:
-        neg = " ".join(("-" + v for v in dtls_versions[:-1]))
+        neg = " ".join("-" + v for v in dtls_versions[:-1])
         text = re.sub(r'\bmin_dtls_version = ' + dtls_versions[-1] + r'\b',
                       f'protocol@TLS = {neg}' if neg else '', text)
         dtls_versions.pop()
@@ -292,13 +317,11 @@ def preprocess_text(text):
 
     tls_versions = list(alg_lists.TLS_PROTOCOLS[::-1])
     while tls_versions:
-        neg = " ".join(("-" + v for v in tls_versions[:-1]))
+        neg = " ".join("-" + v for v in tls_versions[:-1])
         text = re.sub(r'\bmin_tls_version = ' + tls_versions[-1] + r'\b',
                       f'protocol@TLS = {neg}' if neg else '', text)
         tls_versions.pop()
-    text = re.sub(r'\bmin_tls_version = 0\b', '', text)
-
-    return text
+    return re.sub(r'\bmin_tls_version = 0\b', '', text)
 
 
 # Finally, constructing a policy
@@ -315,9 +338,11 @@ class ScopedPolicy:
     >>> ScopedPolicy(parse_line('min_dh_size=2048')).integers['min_dh_size']
     2048
     """
+
     def __init__(self, directives, relevant_scopes=None):
         relevant_scopes = relevant_scopes or set()
         self.integers = INT_DEFAULTS.copy()
+        self.enums = {k: v[0] for k, v in ENUMS.items()}
         self.enabled = {prop_name: [] for prop_name in alg_lists.ALL}
 
         for directive in directives:
@@ -341,9 +366,11 @@ class ScopedPolicy:
                         e for e in self.enabled[directive.prop_name]
                         if e != directive.value
                     ]
-                else:
-                    assert directive.operation == Operation.SET_INT
+                elif directive.operation == Operation.SET_INT:
                     self.integers[directive.prop_name] = directive.value
+                else:
+                    assert directive.operation == Operation.SET_ENUM
+                    self.enums[directive.prop_name] = directive.value
         assert len(self.enabled) == len(set(self.enabled))
 
         self.disabled = {prop_name: [e for e in alg_list
@@ -386,7 +413,7 @@ class UnscopedCryptoPolicy:
 
     def __init__(self, policy_name, *subpolicy_names, policydir=None):
         self.policydir = policydir
-        self.policyname = ':'.join((policy_name,) + subpolicy_names)
+        self.policyname = ':'.join((policy_name, *subpolicy_names))
 
         self.lines = []
 
@@ -421,7 +448,7 @@ class UnscopedCryptoPolicy:
             syntax_check_line(l, warn=True)
         for l in lines:  # crash
             syntax_check_line(l)
-        return sum([parse_line(l) for l in lines], [])
+        return [x for l in lines for x in parse_line(l)]
 
     def __str__(self):
         def fmt(key, value):
@@ -435,14 +462,17 @@ class UnscopedCryptoPolicy:
         s += '# it is provided for review convenience only.\n'
         s += '#\n'
         s += '# Baseline values for all scopes:\n'
-        generic_all = {**generic_scoped.enabled, **generic_scoped.integers}
+        generic_all = {**generic_scoped.enabled,
+                       **generic_scoped.integers,
+                       **generic_scoped.enums}
         for prop_name, value in generic_all.items():
             s += fmt(prop_name, value)
         anything_scope_specific = False
         for scope_name, scope_set in DUMPABLE_SCOPES.items():
             specific_scoped = self.scoped(scopes=scope_set)
             specific_all = {**specific_scoped.enabled,
-                            **specific_scoped.integers}
+                            **specific_scoped.integers,
+                            **specific_scoped.enums}
             for prop_name, value in specific_all.items():
                 if value != generic_all[prop_name]:
                     if not anything_scope_specific:
